
/**
 *
 * \file     km_dsfmt_jump.c
 * \brief    interface between dSFMT-jump and external codes (C,C++,FORTRAN)
 * \authoers Toshiyuki Imamura (TI)
 * \date     2014/03/09 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */


#include "km_dsfmt_jump.h"
#include <stdlib.h>

#define DEF_RAND_JUMP_FILE_PATH   "/etc/kmath/random/jump"
#define DEF_RAND_JUMP_FILE_PREFIX "file"


void
km_dsfmt_jump_do_(dsfmt_t* dsfmt, 
		  const int* seed,
		  const int* irank,
		  const int* nrank,
		  int* ret)
{
  const char* envPath   = getenv("KMATH_RAND_JUMP_FILE_PATH");
  const char* envPrefix = getenv("KMATH_RAND_JUMP_FILE_PREFIX");

  char jumpFile[1024];
  sprintf(jumpFile, "%s/%s_%05d", 
	  envPath   != NULL ? envPath   : DEF_RAND_JUMP_FILE_PATH, 
	  envPrefix != NULL ? envPrefix : DEF_RAND_JUMP_FILE_PREFIX,
	  *seed);

  FILE* fp = fopen(jumpFile, "r");
  if (fp == NULL)
    {
      *ret = -1;
      return ;
    }

  int max_ranks;
  if (fread(&max_ranks, sizeof(int), 1, fp) != 1)
    {
      fclose(fp);
      *ret = -3;
      return ;
    }
  if (*nrank > max_ranks)
    {
      fclose(fp);
      *ret = -2;
      return ;
    }

  if (fseek(fp, sizeof(dsfmt_t) * *irank, SEEK_CUR) != 0)
    {
      fclose(fp);
      *ret = -3;
      return ;
    }

  if (fread(dsfmt, sizeof(dsfmt_t), 1, fp) != 1)
    {
      fclose(fp);
      *ret = -3;
      return ;
    }

  fclose(fp);
  *ret = 0;
}
