
/**
 *
 * \file     km_dsfmt_jump.cpp
 * \brief    interface functions of dSFMT-jump
 * \authoers Toshiyuki Imamura (TI)
 * \date     2014/03/09 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */


#include "km_dsfmt_jump.h"
#include "km_dsfmt_poly.h"

#include "dSFMT-calc-jump.hpp"
#include "dSFMT-jump.h"

#include <NTL/GF2X.h>
#include <NTL/ZZ.h>
#include <stdlib.h>
#include <string.h>

using namespace dsfmt;
using namespace NTL;
using namespace std;

static bool   g_gen_init = false;
static int    g_gen_max_ranks;
static int    g_gen_rand_range;
static char   g_gen_install_dir[512];
static char   g_gen_file_prefix[512];
static string g_gen_range_str;


extern "C" void
km_dsfmt_jump_gen_init_(const int*  max_ranks,
			const int*  rand_range,
			const char* install_dir,
			const char* file_prefix)
{
  if (g_gen_init)
    return ;

  g_gen_max_ranks = *max_ranks;
  g_gen_rand_range  = *rand_range;
  strcpy(g_gen_install_dir, install_dir);
  strcpy(g_gen_file_prefix, file_prefix);


  ZZ range;
  range = 1;
  for(int i = 0; i < g_gen_rand_range; ++i)
    range *= 2;


  GF2X poly;
  stringtopoly(poly, km_dsfmt_poly);


  calc_jump(g_gen_range_str, range, poly);


  g_gen_init = true;

}


extern "C" void
km_dsfmt_jump_gen_(const int* seed, int* ret)
{
  if (!g_gen_init)
    return ;

  *ret = 0;

  dsfmt_t dsfmt;
  dsfmt_init_gen_rand(&dsfmt, (uint32_t)*seed);

  char filename[256];
  sprintf(filename, "%s/%s_%05d", g_gen_install_dir, g_gen_file_prefix, *seed);
  printf("open dSFMT serialized file [%s]\n", filename);

  FILE* fp = fopen(filename, "w");
  if (fp == NULL)
    {
      printf("ERROR : can not open file.\n");
      *ret = -1;
      return ;
    }

  fwrite(&g_gen_max_ranks, sizeof(int), 1, fp);

  fwrite(&dsfmt, sizeof(dsfmt_t), 1, fp);

  for(int rank = 0; rank < g_gen_max_ranks-1; ++rank)
    {
      dSFMT_jump(&dsfmt, g_gen_range_str.c_str());
      fwrite(&dsfmt, sizeof(dsfmt_t), 1, fp);
    }

  fclose(fp);
}
