
/**
 *
 * \file     test.c
 * \brief    test of KMATH random C module
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/02/04 (NT)
 * \date     2014/01/23 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#include "kmath_random.h"
#include <stdio.h>

int main(int argc, char** argv)
{
  int comm_rank, i;
  void* h;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);

  MPI_Comm mpi_comm_country;
  int comm_country_rank;
  int color;

  color = comm_rank % 2;

  MPI_Comm_split(MPI_COMM_WORLD, color, comm_rank, &mpi_comm_country);
  MPI_Comm_rank (mpi_comm_country, &comm_country_rank);

  /* initialize random */
  h = KMATH_Random_init(mpi_comm_country);
  if (h == NULL)
    {
      printf("Failed to initialize. rank:%d\n", comm_country_rank);
      MPI_Finalize();
      return -1;
    }

  /* output random number */
  char file[128];
  sprintf(file, "out_com%04d_rnk%04d", color, comm_country_rank);
  FILE* fp = fopen(file,"w");

  KMATH_Random_seed(h, 1);

  for(i = 0; i < 10; ++i)
    {
      double v;
      KMATH_Random_get(h, &v);
      fprintf(fp, " %17.15f\n", v);
    }

  fclose(fp);

  /* serialize random status */
  sprintf(file, "ser_comm%04d", color);
  KMATH_Random_serialize(h, file);

  /* shutdown random */
  KMATH_Random_finalize(h);

  MPI_Finalize();
  return 0;
}
