/**
 *
 * \file     test_c_io.c
 * \brief    serialize test of KMATH random C module
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/12/17 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#include "kmath_random.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
  int comm_rank, i, mode, count;
  void* h;

  if (argc < 3)
    return -1;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);

  h = KMATH_Random_init(MPI_COMM_WORLD);
  if (h == NULL)
    {
      printf("Failed to initialize. rank:%d\n", comm_rank);
      goto ERR;
    }

  mode = atoi(argv[1]);
  count = atoi(argv[2]);

  FILE* fp;
  char ofile[256];

  switch(mode)
    {
    case 1:

      KMATH_Random_seed(h, 1);

      sprintf(ofile, "out_c_io_1_%04d", comm_rank);
      fp = fopen(ofile, "w");

      for(i = 0; i < count; ++i)
	{
	  double v;
	  KMATH_Random_get(h, &v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      fclose(fp);
      break;
    
    case 2:

      KMATH_Random_seed(h, 1);

      sprintf(ofile, "out_c_io_2_%04d", comm_rank);
      fp = fopen(ofile, "w");

      for(i = 0; i < count; ++i)
	{
	  double v;
	  KMATH_Random_get(h, &v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      if (KMATH_Random_serialize(h, "./data_c_io") != 0)
	{
	  printf("Write ERROR\n");
	  break;
	}

      fclose(fp);
      break;

    case 3:

      sprintf(ofile, "out_c_io_2_%04d", comm_rank);
      fp = fopen(ofile, "a");

      if (KMATH_Random_deserialize(h, "./data_c_io") != 0)
	{
	  printf("Read ERROR\n");
	  break;
	}

      for(i = 0; i < count; ++i)
	{
	  double v;
	  KMATH_Random_get(h, &v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      fclose(fp);
      break;
    
    }


  KMATH_Random_finalize(h);

 ERR:
  MPI_Finalize();

  return 0;
}
