/**
 *
 * \file     test_cpp_io.cpp
 * \brief    serialize test of KMATH random CPP module
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/12/17 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#include "kmath_random.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
  KMATH_Random rand;
  int comm_rank, mode, count;

  if (argc < 3)
    return -1;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);

  if (!rand.init(MPI_COMM_WORLD))
    {
      printf("Failed to initialize. rank:%d\n", comm_rank);
      goto ERR;
    }

  mode  = atoi(argv[1]);
  count = atoi(argv[2]);

  FILE* fp;
  char ofile[256];

  switch(mode)
    {
    case 1:

      rand.seed(1);

      sprintf(ofile, "out_cpp_io_1_%04d", comm_rank);
      fp = fopen(ofile, "w");

      for(int i = 0; i < count; ++i)
	{
	  double v;
	  rand.get(v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      fclose(fp);
      break;
    
    case 2:

      rand.seed(1);

      sprintf(ofile, "out_cpp_io_2_%04d", comm_rank);
      fp = fopen(ofile, "w");

      for(int i = 0; i < count; ++i)
	{
	  double v;
	  rand.get(v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      if (rand.serialize("./data_cpp_io") != 0)
	{
	  printf("Write ERROR\n");
	  break;
	}

      fclose(fp);
      break;

    case 3:

      sprintf(ofile, "out_cpp_io_2_%04d", comm_rank);
      fp = fopen(ofile, "a");

      if (rand.deserialize("./data_cpp_io") != 0)
	{
	  printf("Read ERROR\n");
	  break;
	}

      for(int i = 0; i < count; ++i)
	{
	  double v;
	  rand.get(v);
	  fprintf(fp, "Rank:%04d V:%f\n", comm_rank, v);
	}

      fclose(fp);
      break;
    
    }


  rand.finalize();

 ERR:
  MPI_Finalize();

  return 0;
}
