!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Program  test_f90_io
!> @brief   serialize test of KMATH random FORTRAN90 module
!! @authors Toshiyuki Imamura (TI)
!! @date    2013/12/17 (NT)
!
!  (c) Copyright 2013 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8
program test_io

  use kmath_random_mod
  use mpi

  implicit none


  ! local variables
  type(s_km_rand)  :: rand
  double precision :: v
  integer          :: comm_rank, ierr, i, mode, count
  character(200)   :: str


  call MPI_Init(ierr)
  call MPI_Comm_rank(MPI_COMM_WORLD, comm_rank, ierr)

  call KMATH_Random_Init(rand, MPI_COMM_WORLD, ierr)
  if (ierr /= 0) then
    write(6,*) 'Failed to initialize. rank:', comm_rank
    goto 2
  end if
  

  call getarg(1,str)
  read(str,*) mode
  
  call getarg(2,str)
  read(str,*) count


  select case(mode)

  case (1)

    call KMATH_Random_Seed(rand, 1, ierr)

    write(str,'(a,i4.4)') 'out_f90_io_1_', comm_rank
    open(11, &
         file=str, &
         status='replace', &
         form='formatted')

    do i = 1, count

      call KMATH_Random_Get(rand, v, ierr)
      write(11,*) 'Rank:', comm_rank, v

    end do

    close(11)

  case (2)

    call KMATH_Random_Seed(rand, 1, ierr)

    write(str,'(a,i4.4)') 'out_f90_io_2_', comm_rank
    open(11, &
         file=str, &
         status='replace', &
         form='formatted')

    do i = 1, count

      call KMATH_Random_Get(rand, v, ierr)
      write(11,*) 'Rank:', comm_rank, v

    end do

    call KMATH_Random_Serialize(rand, "./data_f90_io", ierr)
    if (ierr /= 0) then
      write(6,*) 'Write ERROR'
      goto 1
    end if

    close(11)

  case (3)

    write(str,'(a,i4.4)') 'out_f90_io_2_', comm_rank
    open(11, &
         file=str, &
         status='old', &
         form='formatted', &
         access='append')

    call KMATH_Random_Deserialize(rand, "./data_f90_io", ierr)
    if (ierr /= 0) then
      write(6,*) 'Read ERROR'
      goto 1
    end if

    do i = 1, count

      call KMATH_Random_Get(rand, v, ierr)
      write(11,*) 'Rank:', comm_rank, v

    end do

    close(11)

  end select


1 call KMATH_Random_Finalize(rand, ierr)

2 call MPI_Finalize(ierr)

  stop

end program test_io
