
/**
 *
 * \file     test.c
 * \brief    test of KMATH random
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/12/17 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#include "kmath_random.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
  int seed = 1;
  if (argc > 1)
    seed = atoi(argv[1]);

  int comm_rank, comm_size, i;
  void* h;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);

  h = KMATH_Random_init(MPI_COMM_WORLD);
  if (h == NULL)
    {
      printf("Failed to initialize. rank:%d\n", comm_rank);
      MPI_Finalize();
      return -1;
    }

  KMATH_Random_seed(h, seed);

  double v[10];
  for(i = 0; i < 10; ++i)
    KMATH_Random_get(h, &v[i]);

  KMATH_Random_finalize(h);

  double* vv = malloc(sizeof(double) * comm_size * 10);

  MPI_Gather(v,  10, MPI_REAL8,
	     vv, 10, MPI_REAL8, 0, MPI_COMM_WORLD);

  if (comm_rank == 0)
    {
      for(i = 0; i < 10 * comm_size; ++i)
	printf("%f\n", vv[i]);
    }

  free(vv);

  MPI_Finalize();
  return 0;
}
