
/**
 *
 * \file     test.c
 * \brief    benchmark of KMATH random
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/12/18 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#include "kmath_random.h"
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

double get_time()
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return tv.tv_sec * 1000000.0 + (double)tv.tv_usec;
}

int main(int argc, char** argv)
{
  int seed = 1;
  if (argc > 1)
    seed = atoi(argv[1]);

  int comm_rank, comm_size;
  void* h;

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);

  double startKmathTime, endKmathTime;
  double startRandTime, endRandTime;
  double v, v0;

  {
    startKmathTime = get_time();

    h = KMATH_Random_init(MPI_COMM_WORLD);
    if (h == NULL)
      {
	printf("Failed to initialize. rank:%d\n", comm_rank);
	MPI_Finalize();
	return -1;
      }

    KMATH_Random_seed(h, seed);

    {
      startRandTime = get_time();

      KMATH_Random_get(h, &v0);

      long long i;
      for(i = 0; i < 1000000000LL; ++i)
	KMATH_Random_get(h, &v);

      endRandTime = get_time();
    }

    KMATH_Random_finalize(h);

    endKmathTime = get_time();

  }

  double rangeKmathTime = endKmathTime   - startKmathTime;
  double rangeRandTime  = endRandTime    - startRandTime;
  double diffTime       = rangeKmathTime - rangeRandTime;

  printf("Rank%05d: KMATH: %f | Rand: %f | Diff: %f | First: %f | Last: %f\n",
	 comm_rank, 
	 rangeKmathTime / 1000000.0,
	 rangeRandTime  / 1000000.0,
	 diffTime       / 1000000.0,
	 v0, v);

  MPI_Finalize();
  return 0;
}
