
/**
 *
 * \file     km_rand_gen_jump.c
 * \brief    generate dSFMT jump file
 * \authoers Toshiyuki Imamura (TI)
 * \date     2013/12/09 (NT)
 * 
 * (c) Copyright 2013 RIKEN. All rights reserved.
 */

#if MPI_PARALLEL
#include <mpi.h>
#endif
#include "km_dsfmt_jump.h"

#include <stdlib.h>
#include <string.h>

int  comm_rank;
int  comm_size;

int  seed_start;
int  seed_end;
int  max_ranks;
int  rand_range;
char install_dir[256];
char file_prefix[256];


void initParams()
{
  seed_start  = 1;
  seed_end    = 1;
  max_ranks   = 1;
  rand_range  = 100;
  strcpy(install_dir, "./jump");
  strcpy(file_prefix, "file");
}


void readParams(int argc, char** argv)
{
  for(int i = 1; i < argc; ++i)
    {
      if (!strcmp(argv[i], "-seed") && i + 1 < argc)
	{
	  seed_start = atoi(argv[i+1]);

	  if (i + 2 < argc)
	    seed_end = atoi(argv[i+2]);
	  else
	    seed_end = seed_start;
	}

      else if (!strcmp(argv[i], "-max_ranks") && i + 1 < argc)
	{
	  max_ranks = atoi(argv[i+1]);
	}

      else if (!strcmp(argv[i], "-rand_range") && i + 1 < argc)
	{
	  rand_range = atoi(argv[i+1]);
	}

      else if (!strcmp(argv[i], "-install_dir") && i + 1 < argc)
	{
	  strcpy(install_dir, argv[i+1]);
	}

      else if (!strcmp(argv[i], "-file_prefix") && i+ 1 < argc)
	{
	  strcpy(file_prefix, argv[i+1]);
	}
    }
}


void printParams()
{
  if (comm_rank != 0)
    return ;

  printf("\n");
  printf("********** km_rand_gen_jump **********\n");
  printf("\n");

  printf("  seed start   = %d\n", seed_start);
  printf("  seed end     = %d\n", seed_end);
  printf("\n");

  printf("  max ranks    = %d\n", max_ranks);
  printf("\n");

  printf("  rand range   = 2^%d\n", rand_range);
  printf("\n");

  printf("  install dir. = %s\n", install_dir);
  printf("\n");

  printf("  file prefix  = %s\n", file_prefix);
  printf("\n");
}


void output()
{
  km_dsfmt_jump_gen_init_(&max_ranks, 
			  &rand_range,
			  install_dir,
			  file_prefix);

  for(int seed = seed_start; seed <= seed_end; seed += comm_size)
    {
      int s = seed + comm_rank;
      int ret;
      if (s <= seed_end)
	{
	  km_dsfmt_jump_gen_(&s, &ret);
	  if (ret != 0)
	    break;
	}
    }
}


int main(int argc, char** argv)
{

#if MPI_PARALLEL
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
#endif


  initParams();

  readParams(argc, argv);

  printParams();

  output();


#if MPI_PARALLEL
  MPI_Finalize();
#endif

  return 0;
}
